//============================================================================
// Name        : acat.cpp
// Author      : Martin Zak
//============================================================================

#include <iostream>
#include <string>
#include <set>

#include "AutomatonFactory.h"
#include "GrammarFactory.h"
#include "AlibException.h"

#include "regexp/RegExpParser.h"

#include "automaton/AutomatonParser.h"
#include "grammar/GrammarParser.h"
#include "grammar/RightRegularGrammar.h"

#include "sax/SaxInterface.h"

#include "sax/ParserException.h"

#include <exception>

using namespace std;
using namespace automaton;
using namespace regexp;
using namespace alib;
using namespace sax;
using namespace grammar;

void getAutomaton(list<Token>& tokens, bool complexTypes) {
	UnknownAutomaton automaton = AutomatonParser::parse(tokens);

	if(complexTypes) {
		Automaton* concreateAutomaton = AutomatonFactory::buildAutomaton(automaton);
		concreateAutomaton->toXML(cout);
	} else {
		automaton.toXML(cout);
	}
}

void getGrammar(list<Token>& tokens) {
	UnknownGrammar grammar = GrammarParser::parse(tokens);
	grammar.toXML(cout);
}

void getRegExp(list<Token>& tokens) {
	RegExp regexp = RegExpParser::parse(tokens);
	regexp.toXML(cout);
}

int main(int argc, char** argv) {

	int fileParameterIndex = -1;
	bool complexTypes = false;

	try {
		if (argc > 1) {
			for (int i = 1; i < argc; i++) {
				if (string("-h").compare(argv[i]) == 0) {
					cout << "Automaton parsing.\nUsage: catPDA [automaton.xml]\n";
					return -1;
				} else if (string("-c").compare(argv[i]) == 0) {
					complexTypes = true;
				} else {
					if(fileParameterIndex == -1) {
						fileParameterIndex = i;
					} else {
						throw AlibException("Only one file can be passed as parameter - " + string(argv[i]) + " " + string(argv[fileParameterIndex]));
					}
				}
			}
		}

		list<Token> tokens;

		if(fileParameterIndex != -1) {
			SaxInterface::parseFile(argv[fileParameterIndex],tokens);
		} else {
			string input(istreambuf_iterator<char>(cin), (istreambuf_iterator<char>()));
			SaxInterface::parseMemory(input, tokens);
		}

		if (tokens.front().getData() == "automaton") {
			getAutomaton(tokens, complexTypes);
		} else if (tokens.front().getData() == "grammar") {
			getGrammar(tokens);
		} else if (tokens.front().getData() == "regexp") {
			getRegExp(tokens);
		} else {
			throw AlibException("Expected root tag automaton, grammar or regexp. Read: " + tokens.front().getData());
		}

	} catch (AlibException& e) {
		cout << e.what() << endl;
		return -1;
	}

	cout.flush();
}
