/*
 * DotConverter.cpp
 *
 *  Created on: Apr 1, 2013
 *      Author: martin
 */

#include "DotConverter.h"
#include "automaton/State.h"

#include "automaton/Automaton.h"

#include <set>
#include <map>
#include <list>
#include <utility>
#include <vector>
#include <typeinfo>

using namespace std;
using namespace automaton;

void DotConverter::convert(const automaton::Automaton& a, std::ostream& out) {
	out << "digraph automaton {\n";
	out << "rankdir=LR;\n";
	int cnt = 1;

	//Map states to indices
	map<State, int> states;
	for (set<State>::iterator it = a.getStates().begin(); it != a.getStates().end(); it++) {
		states.insert(pair<State, int>((*it), cnt++));
	}

	//Print final states
	for (set<State>::const_iterator fsi = a.getFinalStates().begin(); fsi != a.getFinalStates().end(); fsi++) {
		out << "node [shape = doublecircle, label=\"" << fsi->getName() << "\"]; " << states.find(*fsi)->second
				<< ";\n";
	}

	//Print nonfinal states
	for (map<State, int>::const_iterator it = states.begin(); it != states.end(); it++) {
		if (a.getFinalStates().find(it->first) == a.getFinalStates().end()) {
			out << "node [shape = circle, label=\"" << it->first.getName() << "\" ]; " << it->second << ";\n";
		}
	}

	//Mark initial states
	out << "node [shape = plaintext, label=\"start\"]; 0; \n";
	for (set<State>::const_iterator it = a.getInitialStates().begin(); it != a.getInitialStates().end(); it++) {
		out << "0 -> " << states.find(*it)->second << ";\n";
	}

	try {
		const FSM& fsm = dynamic_cast<const FSM&>(a);
		transitions(fsm, states, out);
		out << "}";
		return;
	} catch (const std::bad_cast& e) {
	}

	try {
		const PDA& pda = dynamic_cast<const PDA&>(a);
		transitions(pda, states, out);
		out << "}";
		return;
	} catch (const std::bad_cast& e) {
	}

	try {
		const TM& tm = dynamic_cast<const TM&>(a);
		transitions(tm, states, out);
		out << "}";
		return;
	} catch (const std::bad_cast& e) {
	}

}

void DotConverter::transitions(const automaton::FSM& fsm, const std::map<automaton::State, int>& states,
		std::ostream& out) {

	map<pair<int, int>, string> transitions;
	const set<automaton::TransitionFSM>& t = fsm.getTransitions();

	//put transitions from automaton to "transitions"
	set<automaton::TransitionFSM>::const_iterator it = t.begin();
	while (it != t.end()) {
		string symbol;
		if (it->getInput().getSymbol().compare("") == 0) {
			symbol = "&epsilon;";
		} else {
			symbol = it->getInput().getSymbol();
		}

		pair<int, int> key(states.find(it->getFrom())->second, states.find(it->getTo())->second);
		map<pair<int, int>, string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(pair<pair<int, int>, string>(key, symbol));
		} else {
			mapit->second += "," + symbol;
		}

		it++;
	}

	//print the map
	map<pair<int, int>, string>::iterator it2 = transitions.begin();
	while (it2 != transitions.end()) {
		out << it2->first.first << " -> " << it2->first.second;
		out << "[label=\"" << it2->second << "\"]\n";
		it2++;
	}
}
void DotConverter::transitions(const automaton::PDA& pda, const std::map<automaton::State, int>& states,
		std::ostream& out) {
	map<pair<int, int>, string> transitions;
	const set<automaton::TransitionPDA>& t = pda.getTransitions();

	//put transitions from automaton to "transitions"
	set<automaton::TransitionPDA>::const_iterator it = t.begin();
	while (it != t.end()) {
		string symbol;

		//input symbol
		if (it->getInput().getSymbol().compare("") == 0) {
			symbol = "&epsilon;";
		} else {
			symbol = it->getInput().getSymbol();
		}

		symbol += ",";

		//Pop part
		if (it->getPop().size() == 0) {
			symbol += " &epsilon;";
		} else {
			for (list<Symbol>::const_iterator symb = it->getPop().begin(); symb != it->getPop().end(); symb++) {
				symbol += " " + symb->getSymbol();
			}

		}

		symbol += "/";

		//Push part
		if (it->getPush().size() == 0) {
			symbol += "&epsilon;";
		} else {
			for (list<Symbol>::const_iterator symb = it->getPush().begin(); symb != it->getPush().end(); symb++) {
				symbol += symb->getSymbol() + " ";
			}

		}

		//Insert into map
		pair<int, int> key(states.find(it->getFrom())->second, states.find(it->getTo())->second);
		map<pair<int, int>, string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(pair<pair<int, int>, string>(key, symbol));
		} else {
			mapit->second += "\\n" + symbol;
		}

		it++;
	}

	//print the map
	map<pair<int, int>, string>::iterator it2 = transitions.begin();
	while (it2 != transitions.end()) {
		out << it2->first.first << " -> " << it2->first.second;
		out << "[label=\"" << it2->second << "\"]\n";
		it2++;
	}
}

void DotConverter::transitions(const automaton::TM& tm, const std::map<automaton::State, int>& states,
		std::ostream& out) {
	map<pair<int, int>, string> transitions;
	const set<automaton::TransitionTM>& t = tm.getTransitions();

	//put transitions from automaton to "transitions"
	set<automaton::TransitionTM>::const_iterator it = t.begin();
	while (it != t.end()) {
		string symbol;

		//input symbol
		symbol = it->getInput().getSymbol();
		symbol += "/";
		symbol += it->getOutput().getSymbol();
		symbol += " ";
		symbol += (std::string[] ) { "&larr;", "&rarr;", "&times;" } [it->getShift()];

		//Insert into map
		pair<int, int> key(states.find(it->getFrom())->second, states.find(it->getTo())->second);
		map<pair<int, int>, string>::iterator mapit = transitions.find(key);

		if (mapit == transitions.end()) {
			transitions.insert(pair<pair<int, int>, string>(key, symbol));
		} else {
			mapit->second += "\\n" + symbol;
		}

		it++;
	}

	//print the map
	map<pair<int, int>, string>::iterator it2 = transitions.begin();
	while (it2 != transitions.end()) {
		out << it2->first.first << " -> " << it2->first.second;
		out << "[label=\"" << it2->second << "\"]\n";
		it2++;
	}
}
