#include "IDPDADeterminizer.h"

namespace determinization {

set<State> IDPDADeterminizer::findToStates(const set<State>& nidpdaStates, const Symbol& input, list<Symbol>* stackPop, list<Symbol>* stackPush) {
    const set<TransitionPDA>& nidpdaTransitions = this->nondeterministicIDPDA->getTransitions();
    set<State> targetStates;
    for (set<State>::iterator state = nidpdaStates.begin(); state != nidpdaStates.end(); state++) {
        for (set<TransitionPDA>::iterator transition = nidpdaTransitions.begin();
                transition != nidpdaTransitions.end();
                transition++) {
            if (transition->getFrom() == *state && transition->getInput() == input) {
                *stackPop = transition->getPop();
                *stackPush = transition->getPush();
                targetStates.insert(transition->getTo());
            }
        }
    }
    return targetStates;
}


IDPDADeterminizer::IDPDADeterminizer(PDA* automaton) {
    this->nondeterministicIDPDA = automaton;
}


PDA* IDPDADeterminizer::determinize() {
    this->deterministicIDPDA = new PDA();
    DeterminizationUtils::copyInputAlphabet(*this->nondeterministicIDPDA, *this->deterministicIDPDA);
    DeterminizationUtils::copyStackAlphabet(*this->nondeterministicIDPDA, *this->deterministicIDPDA);
    map<string, DeterminizationUtils::StateData> states;
    
    const set<State>& nidpdaInitialStates = this->nondeterministicIDPDA->getInitialStates();
    const State& initialState = DeterminizationUtils::getOrCreateState(nidpdaInitialStates, states, *this->deterministicIDPDA);
    
    while (true) {
        bool allStatesMarked = true;
        for (map<string, DeterminizationUtils::StateData>::iterator stateIter = states.begin();
                stateIter != states.end();
                stateIter++) {
            allStatesMarked &= stateIter->second.isMarked;
        }
        if (allStatesMarked) {
            break;
        }
        
        DeterminizationUtils::StateData* unmarkedStateData;
        for (map<string, DeterminizationUtils::StateData>::iterator stateIter = states.begin();
                stateIter != states.end();
                stateIter++) {
            DeterminizationUtils::StateData& stateData = stateIter->second;
            if (!stateData.isMarked) {
                unmarkedStateData = &stateData;
                break;
            }
        }
        
        const set<Symbol>& nidpdaInputAlphabet = this->nondeterministicIDPDA->getInputAlphabet();
        for (set<Symbol>::iterator input = nidpdaInputAlphabet.begin(); input != nidpdaInputAlphabet.end(); input++) {
            list<Symbol> stackPop;
            list<Symbol> stackPush;
            const set<State>& nidpdaToStates = this->findToStates(unmarkedStateData->originalStates, *input, &stackPop, &stackPush);
            if (nidpdaToStates.size() > 0) {
                const State& targetState = DeterminizationUtils::getOrCreateState(nidpdaToStates, states, *this->deterministicIDPDA);
                TransitionPDA transition(unmarkedStateData->state, *input, targetState, stackPop, stackPush);
                this->deterministicIDPDA->addTransition(transition);
            }
        }
        
        unmarkedStateData->isMarked = true;
    }
    
    this->deterministicIDPDA->addInitialState(initialState);
    
    for (map<string, DeterminizationUtils::StateData>::iterator stateIter = states.begin();
            stateIter != states.end();
            stateIter++) {
        const DeterminizationUtils::StateData& stateData = stateIter->second;
        if (DeterminizationUtils::containSomeFinalStateOfAutomaton(stateData.originalStates, *this->nondeterministicIDPDA)) {
            this->deterministicIDPDA->addFinalState(stateData.state);
        }
    }
    
    return this->deterministicIDPDA;
}

} /* namespace determinization */
