#ifndef VPADETERMINIZER_H_
#define VPADETERMINIZER_H_

#include <iostream>
#include <math.h>
#include <string>
#include <set>
#include <map>
#include <vector>

#include "automaton/PDA/PDA.h"
#include "automaton/Transition.h"
#include "automaton/exception/AutomatonException.h"
#include "alphabet/Symbol.h"
#include "DeterminizationUtils.h"

using namespace std;
using namespace automaton;
using namespace alphabet;

namespace determinization {

class VPADeterminizer {
protected:
    struct StatesPair {
        State state1;
        State state2;
        
        StatesPair(const State& state1, const State& state2) : state1(state1), state2(state2) { }
        
        bool operator < (const StatesPair& other) const {
            return state1 < other.state1 || state1 == other.state1 && state2 < other.state2;
        }
        
        bool operator == (const StatesPair& other) const {
            return state1 == other.state1 && state2 == other.state2;
        }
        
        bool operator != (const StatesPair& other) const {
            return state1 != other.state1 || state2 != other.state2;
        }
    };
    
    struct SComponent {
        set<StatesPair> statesPairs;
        SComponent() {}
        SComponent(const set<StatesPair>& statesPairs) : statesPairs(statesPairs) {}
    };
    
    struct RComponent {
        set<State> states;
        RComponent() {}
        RComponent(const set<State>& states) : states(states) {}
    };
    
    struct StateData {
        State state;
        SComponent s;
        RComponent r;
        bool isMarked;
        
        StateData(const State& state, const SComponent& s, const RComponent& r) : s(s), r(r), state(state) {
            this->isMarked = false;
        }
    };
    
    struct PartialStackData {
        SComponent s;
        RComponent r;
        
        PartialStackData(const SComponent& s, const RComponent& r) : s(s), r(r) {}
    };
    
    PDA* nondeterministicVPA;
    PDA* deterministicVPA;
    map<string, StateData> states;
    vector<PartialStackData> partialStackAlphabet;
    set<Symbol> internalSymbols;
    set<Symbol> callSymbols;
    set<Symbol> returnSymbols;
    
    string buildStateName(const SComponent& s, const RComponent& r);
    const State& getOrCreateState(const SComponent& s, const RComponent& r);
    const SComponent getSComponentWithStatesIdentity();
    void divideInputAlphabet();
    string buildStackSymbolName(const SComponent& s, const RComponent& r, const Symbol& input);
    void buildStackAlphabet();
    
public:
    static const Symbol BOTTOM_OF_STACK_SYMBOL;
    
    VPADeterminizer(PDA* automaton);
    PDA* determinize();
    
};

} /* namespace determinization */
#endif /* VPADETERMINIZER_H_ */