/*
 0 * GrammarFactory.cpp
 *
 *  Created on: Nov 5, 2013
 *      Author: martin
 */

#include "GrammarFactory.h"

#include "sax/SaxInterface.h"
#include "grammar/GrammarParser.h"

namespace grammar {

UnknownGrammar GrammarFactory::fromFile(const string& filename) {
	std::list<Token> tokens;
	SaxInterface::parseFile(filename, tokens);
	return parse(tokens);
}

UnknownGrammar GrammarFactory::fromString(const string& str) {
	list<Token> tokens;
	SaxInterface::parseMemory(str, tokens);
	return parse(tokens);
}

UnknownGrammar GrammarFactory::parse(list<sax::Token> tokens) {
	GrammarParser parser;
	return parser.parse(tokens);
}

RightRegularGrammar GrammarFactory::buildRightRegularGrammar(const Grammar& grammar) {
	RightRegularGrammar regularGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		regularGrammar.addRule(rule);
	}

	return regularGrammar;
}

LeftRegularGrammar GrammarFactory::buildLeftRegularGrammar(const Grammar& grammar) {
	LeftRegularGrammar regularGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		regularGrammar.addRule(rule);
	}

	return regularGrammar;
}

LeftLinearGrammar GrammarFactory::buildLeftLinearGrammar(const Grammar& grammar) {
	LeftLinearGrammar linearGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		linearGrammar.addRule(rule);
	}

	return linearGrammar;
}

RightLinearGrammar GrammarFactory::buildRightLinearGrammar(const Grammar& grammar) {
	RightLinearGrammar linearGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		linearGrammar.addRule(rule);
	}

	return linearGrammar;
}

ContextFreeGrammar GrammarFactory::buildContextFreeGrammar(const Grammar& grammar) {
	ContextFreeGrammar contextFreeGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		contextFreeGrammar.addRule(rule);
	}

	return contextFreeGrammar;
}

ContextSensitiveGrammar GrammarFactory::buildContextSensitiveGrammar(const Grammar& grammar) {
	ContextSensitiveGrammar contextSensitiveGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		contextSensitiveGrammar.addRule(rule);
	}

	return contextSensitiveGrammar;
}

UnrestrictedGrammar GrammarFactory::buildUnrestrictedGrammar(const Grammar& grammar) {
	UnrestrictedGrammar unrestrictedGrammar(grammar.getNonTerminalSymbols(), grammar.getTerminalSymbols(),
			grammar.getStartSymbol());

	for (auto rule : grammar.getRules()) {
		unrestrictedGrammar.addRule(rule);
	}

	return unrestrictedGrammar;
}

} /* namespace grammar */

