/*
 * GrammarFactory.h
 *
 *  Created on: Nov 5, 2013
 *      Author: Martin Zak
 */

#ifndef GRAMMARFACTORY_H_
#define GRAMMARFACTORY_H_

#include "sax/Token.h"
#include "grammar/UnknownGrammar.h"
#include "grammar/RightRegularGrammar.h"
#include "grammar/LeftRegularGrammar.h"
#include "grammar/RightLinearGrammar.h"
#include "grammar/LeftLinearGrammar.h"
#include "grammar/ContextFreeGrammar.h"
#include "grammar/ContextSensitiveGrammar.h"

#include "grammar/UnrestrictedGrammar.h"

namespace grammar {

using namespace std;
using namespace sax;

/**
 * Builds specific grammars from UnknownGrammar.
 */
class GrammarFactory {
public:
	/**
	 * Parses the XML in file and returns UnknownGrammar.
	 * @param filename path to the file
	 * @return UnknownGrammar
	 */
	static UnknownGrammar fromFile(const string& filename);

	/**
	 * Parses the XML and returns UnknownGrammar.
	 * @param str string containing the XML
	 * @return UnknownGrammar
	 */
	static UnknownGrammar fromString(const string& str);

	/**
	 * Tries to build RightRegularGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when RightRegularGrammar cannot be build from source grammar
	 */
	static RightRegularGrammar buildRightRegularGrammar(const Grammar& grammar);

	/**
	 * Tries to build LeftRegularGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when LeftRegularGrammar cannot be build from source grammar
	 */
	static LeftRegularGrammar buildLeftRegularGrammar(const Grammar& grammar);

	/**
	 * Tries to build LeftLinearGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when LeftLinearGrammar cannot be build from source grammar
	 */
	static LeftLinearGrammar buildLeftLinearGrammar(const Grammar& grammar);

	/**
	 * Tries to build RightLinearGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when RightLinearGrammar cannot be build from source grammar
	 */
	static RightLinearGrammar buildRightLinearGrammar(const Grammar& grammar);

	/**
	 * Tries to build ContextFreeGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when ContextFreeGrammar cannot be build from source grammar
	 */
	static ContextFreeGrammar buildContextFreeGrammar(const Grammar& grammar);

	/**
	 * Tries to build ContextSensitiveGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when ContextSensitiveGrammar cannot be build from source grammar
	 */
	static ContextSensitiveGrammar buildContextSensitiveGrammar(const Grammar& grammar);

	/**
	 * Tries to build UnrestrictedGrammar from any grammar.
	 * @param grammar source grammar
	 * @return grammar created from source grammar
	 * @throws GrammarException when UnrestrictedGrammar cannot be build from source grammar
	 */
	static UnrestrictedGrammar buildUnrestrictedGrammar(const Grammar& grammar);

protected:
	/**
	 * Parses the UnknownGrammar from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed UnknownGrammar
	 */
	static UnknownGrammar parse(list<Token> tokens);
};

} /* namespace grammar */
#endif /* GRAMMARFACTORY_H_ */
