/*
 * ContextFreeGrammar.h
 *
 *  Created on: Nov 17, 2013
 *      Author: martin
 */

#ifndef CONTEXTFREEGRAMMAR_H_
#define CONTEXTFREEGRAMMAR_H_

#include "Grammar.h"

namespace grammar {

/**
 * Context-free grammar. Type 2 in Chomsky hierarchy.
 */
class ContextFreeGrammar: public Grammar {
private:
	/**
	 * Checks that left side of the rule contains one symbol which is nonterminal.
	 * @param leftSide left side of the rule
	 * @return true when left side of the rule contains one nonterminal symbol, false otherwise
	 */
	bool checkLeftSide(const list<Symbol>& leftSide) const;

	/**
	 * Checks that symbols on the right side of the rule are all terminal or nonterminal.
	 * @param rightSide right side of the rule
	 * @return true when right side of the rule contains only terminal and nonterminal symbols, false otherwise
	 */
	bool checkRightSide(const list<Symbol>& rightSide) const;
protected:
	/**
	 * @copydoc Grammar::isValidRule(const Rule&) const
	 */
	bool isValidRule(const Rule& rule) const;
public:
	ContextFreeGrammar();
	ContextFreeGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
			const Symbol& startSymbol);
};

} /* namespace grammar */
#endif /* CONTEXTFREEGRAMMAR_H_ */
