/*
 * LeftLinearGrammar.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "LeftLinearGrammar.h"

namespace grammar {

LeftLinearGrammar::LeftLinearGrammar() :
		Grammar() {
}

LeftLinearGrammar::LeftLinearGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
		const Symbol& startSymbol) :
		Grammar(nonTerminalSymbols, terminalSymbols, startSymbol) {
}

bool LeftLinearGrammar::isValidRule(const Rule& rule) const {
	return checkLeftSide(rule.getLeftSide()) && checkRightSide(rule.getRightSide());
}

bool LeftLinearGrammar::checkLeftSide(const list<Symbol>& leftSide) const {
	if (leftSide.size() != 1) {
		return false;
	}

	if (nonTerminalSymbols.find(leftSide.front()) == nonTerminalSymbols.end()) {
		return false;
	}

	return true;
}

bool LeftLinearGrammar::checkRightSide(const list<Symbol>& rightSide) const {
	if (rightSide.size() == 0) {
		return true;
	} else if (rightSide.size() == 1) {
		const Symbol& symbol = rightSide.front();

		//check epsilon
		if (symbol.getSymbol() == "") {
			return true;
		}

		//check that symbol exists
		return ((terminalSymbols.find(symbol) != terminalSymbols.end())
				|| (nonTerminalSymbols.find(symbol) != nonTerminalSymbols.end()));
	} else {
		//check if first symbol is nonterminal
		list<Symbol>::const_iterator symbol = rightSide.begin();
		if (nonTerminalSymbols.find(*symbol) == nonTerminalSymbols.end()) {
			return false;
		}

		symbol++;

		//check if following symbols are terminal
		for (; symbol != rightSide.end(); symbol++) {
			if (terminalSymbols.find(*symbol) == terminalSymbols.end()) {
				return false;
			}
		}

		return true;
	}
}

} /* namespace grammar */
