/*
 * RegularGrammar.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "RegularGrammar.h"

namespace grammar {

RegularGrammar::RegularGrammar() :
		Grammar() {
}

RegularGrammar::RegularGrammar(const set<Symbol>& nonTerminalSymbols, const set<Symbol>& terminalSymbols,
		const Symbol& startSymbol) :
		Grammar(nonTerminalSymbols, terminalSymbols, startSymbol) {
}

bool RegularGrammar::checkEpsilonRule(const list<Symbol>& leftSide, const list<Symbol>& rightSide) const {
	if (leftSide.size() != 1 && leftSide.front() != startSymbol) {
		return false;
	}

	//startSymbol cannot be present on the right side of any rule
	for (const auto& rule : rules) {
		for (const auto& symbol : rule.getRightSide()) {
			if (symbol == startSymbol) {
				return false;
			}
		}
	}
	return true;
}

bool RegularGrammar::isEpsilonRule(const Rule& rule) const {
	return rule.getRightSide().size() == 0;
}

bool RegularGrammar::containsEpsilonRule() const {
	list<Symbol> leftSide;
	leftSide.push_back(startSymbol);
	list<Symbol> rightSide;

	Rule epsilonRule(leftSide, rightSide);

	return rules.find(epsilonRule) != rules.end();
}

} /* namespace grammar */

