/*
 * Rule.cpp
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#include "Rule.h"
#include <algorithm>

namespace grammar {

Rule::Rule() {
}

Rule::Rule(const list<Symbol>& leftSide, const list<Symbol>& rightSide) :
		leftSide(leftSide), rightSide(rightSide) {
}

void grammar::Rule::setLeftSide(const list<Symbol>& leftSide) {
	this->leftSide = leftSide;
}

void grammar::Rule::setRightSide(const list<Symbol>& rightSide) {
	this->rightSide = rightSide;
}

const list<Symbol>& grammar::Rule::getLeftSide() const {
	return leftSide;
}

const list<Symbol>& grammar::Rule::getRightSide() const {
	return rightSide;
}

bool Rule::containsSymbol(const Symbol& symbol) const {
	if (find(leftSide.begin(), leftSide.end(), symbol) != leftSide.end()) {
		return true;
	}

	if (find(rightSide.begin(), rightSide.end(), symbol) != rightSide.end()) {
		return true;
	}

	return false;
}

string Rule::toString() const {
	string output;
	output += "[";
	for(auto const& symbol : leftSide) {
		output += " " + symbol.getSymbol();
	}
	output += " -> ";
	for(auto const& symbol : rightSide) {
		output += " " + symbol.getSymbol();
	}

	output += "]";
	return output;
}

bool Rule::operator <(const Rule& other) const {
	if (leftSide.size() != other.leftSide.size()) {
		return leftSide.size() < other.leftSide.size();
	}

	if (rightSide.size() != other.rightSide.size()) {
		return rightSide.size() < other.rightSide.size();
	}

	list<Symbol>::const_iterator left = leftSide.begin();
	list<Symbol>::const_iterator otherLeft = other.leftSide.begin();
	while (left != leftSide.end()) {
		if (*left != *otherLeft) {
			return *left < *otherLeft;
		}
		left++;
		otherLeft++;
	}

	list<Symbol>::const_iterator right = rightSide.begin();
	list<Symbol>::const_iterator otherRight = other.rightSide.begin();
	while (right != rightSide.end()) {
		if (*right != *otherRight) {
			return *right < *otherRight;
		}
		right++;
		otherRight++;
	}

	return false;
}

bool Rule::operator ==(const Rule& other) const {
	return equal(leftSide.begin(), leftSide.end(), other.leftSide.begin())
			&& equal(rightSide.begin(), rightSide.end(), other.rightSide.begin());
}

bool Rule::operator !=(const Rule& other) const {
	return !equal(leftSide.begin(), leftSide.end(), other.leftSide.begin())
			|| !equal(rightSide.begin(), rightSide.end(), other.rightSide.begin());
}

std::ostream& operator<<(std::ostream& out, const Rule& rule) {
	bool first;
	out << " leftSide = [";

	first = true;
	for(list<Symbol>::const_iterator iter = rule.leftSide.begin(); iter != rule.leftSide.end(); iter++) {
		if(!first) out << ", ";
		first = false;
		out << *iter;
	}

	out << "] rightSize = [";

	first = true;
	for(list<Symbol>::const_iterator iter = rule.rightSide.begin(); iter != rule.rightSide.end(); iter++) {
		if(!first) out << ", ";
		first = false;
		out << *iter;
	}
	out << "]";

	return out;
}

} /* namespace grammar */
