/*
 * Rule.h
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#ifndef RULE_H_
#define RULE_H_

#include <list>
#include "../alphabet/Symbol.h"

namespace grammar {

using namespace std;
using namespace alphabet;

/**
 * Represents rewrite rule (production) of a Grammar.
 */
class Rule {
private:
	list<Symbol> leftSide;
	list<Symbol> rightSide;
public:
	Rule();
	Rule(const list<Symbol>& leftSide, const list<Symbol>& rightSide);

	/**
	 * Sets left side of the rule.
	 * @param leftSide list of symbols to set
	 */
	void setLeftSide(const list<Symbol>& leftSide);

	/**
	 * Sets right side of the rule.
	 * @param rightSide list of symbols to set
	 */
	void setRightSide(const list<Symbol>& rightSide);

	/**
	 * @return list of symbols on the left side of the Rule
	 */
	const list<Symbol>& getLeftSide() const;

	/**
	 * @return list of symbols on the right side of the Rule
	 */
	const list<Symbol>& getRightSide() const;

	/**
	 * Checks that Rule contains given symbol.
	 * @param symbol Symbol to find
	 * @return true when Rule contains the symbol, false otherwise
	 */
	bool containsSymbol(const Symbol& symbol) const;

	/**
	 * Converts rule into human readable string.
	 * @return string in format [ leftSide -> rightSide ]
	 */
	string toString() const;

	bool operator <(const Rule& other) const;
	bool operator ==(const Rule& other) const;
	bool operator !=(const Rule& other) const;

	friend std::ostream& operator<<(std::ostream&, const Rule&);
};

} /* namespace grammar */
#endif /* RULE_H_ */
