/*
 * RegExpPrinter.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef REGEXPPRINTER_H_
#define REGEXPPRINTER_H_

#include <ostream>
#include "RegExp.h"
#include "Alternation.h"
#include "Concatenation.h"
#include "Iteration.h"
#include "RegExpSymbol.h"

namespace regexp {

using namespace std;

/**
 * This class contains methods to print XML representation of regular expression to the output stream.
 */
class RegExpPrinter {
protected:
	static const string INDENTATION;
	static void printElement(RegExpElement* element, ostream& out, string prefix);
	static void printContent(list<RegExpElement*>& content, ostream& out, string prefix);
	static void printAlternation(Alternation* alternation, ostream& out, string prefix);
	static void printConcatenation(Concatenation* concatenation, ostream& out, string prefix);

	static void printIteration(Iteration* iteration, ostream& out, string prefix);
	static void printSymbol(RegExpSymbol* symbol, ostream& out, string prefix);

public:
	/**
	 * Prints XML representation of RegExp to the output stream.
	 * @param regexp RegExp to print
	 * @param out output stream to which print the RegExp
	 */
	static void toXML(RegExp& regexp, ostream& out);
};

} /* namespace regexp */
#endif /* REGEXPPRINTER_H_ */
